@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2021, University of Amsterdam
+    Copyright (c)  1985-2024, University of Amsterdam
 			      VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -47,6 +47,7 @@
 #include "pl-fli.h"
 #include "pl-setup.h"
 #include "pl-pro.h"
+#include "pl-comp.h"
 #include <math.h>
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
@@ -221,7 +222,7 @@ freeHeap(void *mem, size_t n)
 		 *******************************/
 
 void
-linger(linger_list** list, void (*unalloc)(void *), void *object)
+linger_always(linger_list** list, void (*unalloc)(void *), void *object)
 { if ( GD->cleaning != CLN_DATA )
   { linger_list *c = allocHeapOrHalt(sizeof(*c));
     linger_list *o;
@@ -444,8 +445,7 @@ Returns `0` if there is no enough space to store this term.
 
 static size_t
 size_frame_term(LocalFrame fr)
-{ GET_LD
-  size_t arity = fr->predicate->functor->arity;
+{ size_t arity = fr->predicate->functor->arity;
   size_t size = 4 + 3 + arity+1;
   size_t i;
 
@@ -838,99 +838,33 @@ newTerm(void)
   return t;
 }
 
-		 /*******************************
-		 *    OPERATIONS ON INTEGERS	*
-		 *******************************/
-
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Translate  a  64-bit  integer  into   a    Prolog   cell.   Uses  tagged
-representation if possible or allocates 64-bits on the global stack.
-
-Return is one of:
-
-	TRUE:		 Success
-	FALSE:		 Interrupt
-	GLOBAL_OVERFLOW: Stack overflow
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
-int
-put_int64(DECL_LD Word at, int64_t l, int flags)
-{ Word p;
-  word r, m;
-  int req;
-
-  r = consInt(l);
-  if ( valInt(r) == l )
-  { *at = r;
-    return TRUE;
-  }
-
-#if SIZEOF_VOIDP == 8
-  req = 3;
-#elif SIZEOF_VOIDP == 4
-  req = 4;
-#else
-#error "FIXME: Unsupported sizeof word"
-#endif
-
-  if ( !hasGlobalSpace(req) )
-  { int rc = ensureGlobalSpace(req, flags);
-
-    if ( rc != TRUE )
-      return rc;
-  }
-  p = gTop;
-  gTop += req;
-
-#if SIZEOF_VOIDP == 8
-  r = consPtr(p, TAG_INTEGER|STG_GLOBAL);
-  m = mkIndHdr(1, TAG_INTEGER);
-
-  *p++ = m;
-  *p++ = l;
-  *p   = m;
-#else
-#if SIZEOF_VOIDP == 4
-  r = consPtr(p, TAG_INTEGER|STG_GLOBAL);
-  m = mkIndHdr(2, TAG_INTEGER);
-
-  *p++ = m;
-#ifdef WORDS_BIGENDIAN
-  *p++ = (word)(l>>32);
-  *p++ = (word)l;
-#else
-  *p++ = (word)l;
-  *p++ = (word)(l>>32);
-#endif
-  *p   = m;
-#else
-#error "FIXME: Unsupported sizeof intptr_t."
-#endif
-#endif
-
-  *at = r;
-  return TRUE;
-}
-
-
 		 /*******************************
 		 *    OPERATIONS ON STRINGS	*
 		 *******************************/
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-To distinguish between byte and wide strings,   the system adds a 'B' or
-'W' in front of the real string. For   a  'W', the following 3 bytes are
-ignored to avoid alignment restriction problems.
+Allocate a  blob of non-word  aligned size,  given the size  in bytes.
+Note that  that last word is  zeroed to make sure  possible padding is
+zero  and canonical  and strings  are 0-terminated  (they may  contain
+internal zeros).
 
-Note that these functions can trigger GC
+TBD: This function  ensures there is at least one  0 byte that follows
+the data.  That  is good for strings,  but not if we want  to use this
+for non-null terminated  data.  If you want to fix  this, take care of
+strings  as  they  live  in   code,  records  and  .QLF  files.   Also
+getCharsString() and getCharsWString() must  be updated to compute the
+correct length.
+
+This function calls allocGlobal() and thus  may cause a stack shift or
+garbage collect.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 Word
-allocString(DECL_LD size_t len)
+globalBlob(DECL_LD size_t len, int tag)
 { size_t lw = (len+sizeof(word))/sizeof(word);
   int pad = (int)(lw*sizeof(word) - len);
   Word p = allocGlobal(2 + lw);
-  word m = mkStrHdr(lw, pad);
+  word m = mkBlobHdr(lw, pad, tag);
 
   if ( !p )
     return NULL;
@@ -943,10 +877,18 @@ allocString(DECL_LD size_t len)
 }
 
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+To distinguish between byte and wide strings, the system adds a 'B' or
+'W'  in  front   of  the  real  string.  For  a   'W',  the  following
+sizeof(wchar_t)-1  bytes are  ignored to  avoid alignment  restriction
+problems.
+
+Note that these functions can trigger GC
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 word
-globalString(size_t len, const char *s)
-{ GET_LD
-  Word p = allocString(len+1);
+globalString(DECL_LD size_t len, const char *s)
+{ Word p = globalBlob(len+1, TAG_STRING);
 
   if ( p )
   { char *q = (char *)&p[1];
@@ -954,7 +896,14 @@ globalString(size_t len, const char *s)
     *q++ = 'B';
     memcpy(q, s, len);
 
-    return consPtr(p, TAG_STRING|STG_GLOBAL);
+    word w = consPtr(p, TAG_STRING|STG_GLOBAL);
+    DEBUG(CHK_SECURE,
+	  { size_t len2;
+	    char *s2 = getCharsString(w, &len2);
+	    assert(len2 == len);
+	    assert(memcmp(s, s2, len) == 0);
+	  });
+    return w;
   }
 
   return 0;
@@ -962,9 +911,8 @@ globalString(size_t len, const char *s)
 
 
 word
-globalWString(size_t len, const pl_wchar_t *s)
-{ GET_LD
-  const pl_wchar_t *e = &s[len];
+globalWString(DECL_LD size_t len, const pl_wchar_t *s)
+{ const pl_wchar_t *e = &s[len];
   const pl_wchar_t *p;
   Word g;
 
@@ -976,7 +924,7 @@ globalWString(size_t len, const pl_wchar_t *s)
   if ( p == e )				/* 8-bit string */
   { unsigned char *t;
 
-    if ( !(g = allocString(len+1)) )
+    if ( !(g = globalBlob(len+1, TAG_STRING)) )
       return 0;
     t = (unsigned char *)&g[1];
     *t++ = 'B';
@@ -986,7 +934,7 @@ globalWString(size_t len, const pl_wchar_t *s)
   { char *t;
     pl_wchar_t *w;
 
-    if ( !(g = allocString((len+1)*sizeof(pl_wchar_t))) )
+    if ( !(g = globalBlob((len+1)*sizeof(pl_wchar_t), TAG_STRING)) )
       return 0;
     t = (char *)&g[1];
     w = (pl_wchar_t*)t;
@@ -1031,7 +979,9 @@ getCharsWString(DECL_LD word w, size_t *len)
 
   s = (char *)&p[1];
   if ( *s != 'W' )
+  { assert(*s == 'B');
     return NULL;
+  }
 
   if ( len )
     *len = ((wn*sizeof(word) - pad)/sizeof(pl_wchar_t)) - 1;
@@ -1080,16 +1030,13 @@ put_double(DECL_LD Word at, double d, int flags)
 }
 
 
-/* valBignum(DECL_LD word w) moved to pl-inline.h */
-
 		 /*******************************
 		 *  GENERIC INDIRECT OPERATIONS	*
 		 *******************************/
 
 int
 equalIndirect(word w1, word w2)
-{ GET_LD
-  Word p1 = addressIndirect(w1);
+{ Word p1 = addressIndirect(w1);
   Word p2 = addressIndirect(w2);
 
   if ( *p1 == *p2 )
@@ -1533,8 +1480,8 @@ tmp_nrealloc(void *mem, size_t req)
 size_t
 tmp_malloc_size(void *mem)
 { if ( mem )
-  { size_t *sp = mem;
-    return sp[-1];
+  { Word sp = mem;
+    return (size_t)sp[-1];
   }
 
   return 0;
@@ -1542,10 +1489,10 @@ tmp_malloc_size(void *mem)
 
 void *
 tmp_malloc(size_t size)
-{ void *mem = malloc(size+sizeof(size_t));
+{ void *mem = malloc(size+sizeof(word));
 
   if ( mem )
-  { size_t *sp = mem;
+  { Word sp = mem;
     *sp++ = size;
 #ifdef O_DEBUG
     memset(sp, 0xFB, size);
@@ -1559,8 +1506,8 @@ tmp_malloc(size_t size)
 
 void *
 tmp_realloc(void *old, size_t size)
-{ size_t *sp = old;
-  size_t osize = *--sp;
+{ Word sp = old;
+  size_t osize = (size_t)*--sp;
   void *mem;
 
 #ifdef O_DEBUG
@@ -1571,7 +1518,7 @@ tmp_realloc(void *old, size_t size)
   }
 #else
   (void)osize;
-  if ( (mem = realloc(sp, size+sizeof(size_t))) )
+  if ( (mem = realloc(sp, size+sizeof(word))) )
   { sp = mem;
     *sp++ = size;
     return sp;
