@@ -232,11 +232,11 @@ static int
 compare_dict_entry(const void *a, const void *b, void *arg)
 { Word p = (Word)a+1;
   Word q = (Word)b+1;
-  WITH_LD(arg)
-  { deRef(p);
-    deRef(q);
-  }
-  return (*p<*q ? -1 : *p>*q ? 1 : 0);
+
+  deRef(p);
+  deRef(q);
+
+  return SCALAR_TO_CMP(*p, *q);
 }
 
 
@@ -667,9 +667,8 @@ get_name_value(DECL_LD Word p, Word name, Word value, mark *m, int flags)
 		 *******************************/
 
 int
-PL_is_dict(term_t t)
-{ GET_LD
-  Word p = valTermRef(t);
+PL_is_dict(DECL_LD term_t t)
+{ Word p = valTermRef(t);
 
   deRef(p);
   if ( isTerm(*p) )
@@ -686,6 +685,15 @@ PL_is_dict(term_t t)
   return FALSE;
 }
 
+API_STUB(int)
+(PL_is_dict)(term_t t)
+( valid_term_t(t);
+  return PL_is_dict(t);
+)
+
+
+/* Turn data into a dict if it is not already a dict.
+ */
 
 static int
 PL_get_dict_ex(term_t data, term_t tag, term_t dict, int flags)
@@ -759,7 +767,7 @@ PL_get_dict_ex(term_t data, term_t tag, term_t dict, int flags)
       assert(rc == -2);
       Undo(m);
       return ( (ex = PL_new_term_ref()) &&
-	       _PL_unify_atomic(ex, dupl) &&
+	       PL_unify_atomic(ex, dupl) &&
 	       PL_error(NULL, 0, NULL, ERR_DUPLICATE_KEY, ex) );
     }
   }					/* TBD: {name:value, ...} */
@@ -803,7 +811,7 @@ cmp_dict_index(DECL_LD const int *ip1, const int *ip2, cmp_dict_index_data *ctx)
   } else
   { if ( isAtom(*p) )
     { if ( isAtom(*q) )
-	rc = compareAtoms(*p, *q);
+	rc = compareAtoms(word2atom(*p), word2atom(*q));
       else
 	rc = CMP_GREATER;
     } else
@@ -910,9 +918,9 @@ pl_for_dict(DECL_LD term_t dict,
 
 /* resortDictsInClause() resorts the contents of dicts in a clause
 
-This predicate is called from pl-wic.c after   reloading a clause from a
-.qlf file or state if pl-wic.c  detected   a  dict  inside the clause. It
-identifies the code ranges that constitute the  k-v pairs in the dict and
+This predicate is called from pl-qlf.c after   reloading a clause from a
+.qlf file or state if pl-qlf.c  detected   a  dict inside the clause. It
+identifies the code ranges that constitute the k-v pairs in the dict and
 re-orders them according to the new atom-handle ordering.
 
 There    is    a    complicating     factor    with    B_FIRSTVAR/B_VAR,
@@ -1262,7 +1270,7 @@ pl_get_dict(term_t PL__t0, int PL__ac, int ex, control_t PL__ctx)
 	  deRef2(&f->arguments[i+1], np);	/* TBD: check type */
 	  if ( unify_ptrs(&f->arguments[i], valTermRef(A3),
 			  ALLOW_GC|ALLOW_SHIFT) &&
-	       _PL_unify_atomic(A1, *np) )
+	       PL_unify_atomic(A1, *np) )
 	  { PL_close_foreign_frame(fid);
 
 	    if ( i+2 < arity )
@@ -1728,8 +1736,10 @@ PL_get_dict_key(atom_t key, term_t dict, term_t value)
   word d;
   Word vp;
 
-  if ( !is_dict_key(key) )
-    return -1;
+  valid_dict_key(key);
+  valid_term_t(dict);
+  valid_term_t(value);
+
   if ( !get_dict_ex(dict, &d, FALSE) )
     return FALSE;
   if ( (vp=dict_lookup_ptr(d, key)) )
