@@ -723,16 +723,13 @@ last_arg:
 
   while(isRef(*p))
   { assert(!is_marked(p));
+    p2 = unRef(*p);
     if ( storage(*p) == STG_LOCAL ) /* PushPtr()/PopPtr() reference? */
     { for(int i=0; i<LD->tmp.top; i++)
       { if ( valTermRef(LD->tmp.h[i]) == p )
-	{ p2 = unRef(*p);
 	  goto deref_ok;
-	}
       }
       printk(context, "Reference to local stack");
-    } else
-    { p2 = unRef(*p);
     }
   deref_ok:
     DEBUG(CHK_HIGHER_ADDRESS,
