@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1999-2023, University of Amsterdam
+    Copyright (c)  1999-2024, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -178,7 +178,7 @@ typedef struct message_queue
 { struct thread_message   *head;	/* Head of message queue */
   struct thread_message   *tail;	/* Tail of message queue */
   uint64_t	       sequence_next;	/* next for sequence id */
-  word		       id;		/* Id of the queue */
+  atom_t	       id;		/* Id of the queue */
   size_t	       size;		/* # terms in queue */
   size_t	       max_size;	/* Max # terms in queue */
   int		       waiting;		/* # waiting threads */
@@ -510,11 +510,17 @@ static inline	PL_local_data_t *acquire_ldata(DECL_LD PL_thread_info_t *info);
 		 *	    FUNCTIONS		*
 		 *******************************/
 
+#ifndef __WINDOWS__
+#define HAVE_PTHREAD_EXIT 1
+#endif
+
 int		exitPrologThreads(void);
 bool		aliasThread(int tid, atom_t type, atom_t name);
-word		pl_thread_create(term_t goal, term_t id,
+foreign_t	pl_thread_create(term_t goal, term_t id,
 				 term_t options);
-word		pl_thread_exit(term_t retcode);
+#if HAVE_PTHREAD_EXIT
+foreign_t	pl_thread_exit(term_t retcode);
+#endif
 
 foreign_t	pl_thread_at_exit(term_t goal);
 int		PL_thread_self(void);
@@ -592,7 +598,8 @@ foreign_t	pl_thread_self(term_t self);
 #define TWF_ASSERT	0x0001		/* Predicate actions */
 #define TWF_RETRACT	0x0002
 
-#define C_STACK_MIN	(100*1024)
+#define C_STACK_MIN	(32*1024)
+#define NEW_C_STACK_MIN (C_STACK_MIN*2)
 
 typedef struct
 { functor_t functor;			/* functor of property */
